
/*
	file: rseq.c
	purpose: reads single sequence input format file 
*/

#include "defs.h" 

/* procedure calls */
void read_seq_file();
static void read_seq_id_str(), read_seq_title();
static void read_seq_data(), read_seq3_data(), store_seq_data();
static int check_sbuf_line();
extern void show_err();


/*************************************************************/
/* Reads one sequence from an input file.
*/
void read_seq_file(fp, rSeq)
FILE *fp;
SEQ_REC *rSeq;
{
	register int ch;
	char sVal[MAX_LINE_LEN];
	char pBuf[MAX_RES_LEN], sBuf[MAX_RES_LEN];

/* look for start of sequence */
	ch = getc(fp);
	while ((ch != EOF) && (ch != INP_SEQ_SCH)) {
		ch = getc(fp);
	}

	if (ch == EOF) {
		rewind(fp);
		read_seq3_data(fp, pBuf, sBuf);
		store_seq_data(rSeq, pBuf, sBuf);
	} else {

/* read sequence id code */
		read_seq_id_str(fp, rSeq);

/* read sequence title */
		read_seq_title(fp, rSeq);

/* read amino acid data */
		read_seq_data(fp, pBuf, sBuf);
		store_seq_data(rSeq, pBuf, sBuf);
	}

}	/* read_seq_file */


/*************************************************************/
/* Reads and verifies the sequence id code of a sequence.
*/
static void read_seq_id_str(fp, rSeq)
FILE *fp;
SEQ_REC *rSeq;
{
	register int len;
	char sVal[MAX_LINE_LEN], aStr[MAX_LINE_LEN];

	if (EOF == fscanf(fp, "%s", aStr)) {
		sprintf(sVal, "Expecting sequence but EOF found.\n");
		show_err(TRUE, "read_seq_id_str", sVal);
	}

	len = strlen(aStr);
	if (len >= MAX_ID_LEN) {
		sprintf(sVal, "Sequence ID code too long at: %s\n", aStr);
		show_err(TRUE, "read_seq_id_str", sVal);
	}

	strcpy(rSeq->id, aStr);

}	/* read_seq_id_str */


/*************************************************************/
/* Reads and verifies the title of a sequence.
*/
static void read_seq_title(fp, rSeq)
FILE *fp;
SEQ_REC *rSeq;
{
	register int i, len;
	char sVal[MAX_LINE_LEN];

	fgets(sVal, MAX_TITLE_LEN-1, fp);

	len = strlen(sVal);
	for (i=0; i<len; i++) {
		if ((sVal[i] != ' ') && (sVal[i] != '\t'))
			break;
	}

	strcpy(rSeq->title, &sVal[i]);

	if (sVal[i] == '\n') {
		sprintf(sVal, "Missing sequence title at: %s", rSeq->id);
		show_err(TRUE, "read_seq_title", sVal);
	}

}	/* read_seq_title */


/*************************************************************/
/* Reads one sequence from an input file checking that input
** data is legit.
*/
static void read_seq3_data(fp, pBuf, sBuf)
FILE *fp;
char *pBuf, *sBuf;
{
	register int nd, i, ch, len, pLen, sLen;
	char line[MAX_LINE_LEN], newLine[MAX_LINE_LEN];
	char sVal[MAX_LINE_LEN], res[MAX_LINE_LEN];

	pLen = sLen = 0;
	while (TRUE) {
		nd = fscanf(fp, "%3s", res);
		if (nd == EOF)
			break;
		if (res[0] != COMMENT) {
			upper_case(res);
			ch = pres_3to1(res);
			if (ch == ERROR_FLAG) {
				sprintf(sVal, "Illegal 3 letter amino acid: %s", res);
				show_err(TRUE, "read_seq3_data", sVal);
			}
			pBuf[pLen++] = ch + 'A';
			sBuf[sLen++] = 'C';
		}
		if (pLen > MAX_RES_LEN) { /* too many residues */
			sprintf(sVal, "Sequence has > %d amino acids",
				MAX_RES_LEN);
			show_err(TRUE, "read_seq_data", sVal);
		}
	}

	pBuf[pLen] = '\0';
	sBuf[pLen] = '\0';

}	/* read_seq3_data */


/*************************************************************/
/* Reads one sequence from an input file checking that input
** data is legit.
*/
static void read_seq_data(fp, pBuf, sBuf)
FILE *fp;
char *pBuf, *sBuf;
{
	register int i, ch, len, pLen, sLen;
	char line[MAX_LINE_LEN], newLine[MAX_LINE_LEN];
	char sVal[MAX_LINE_LEN];

	pLen = sLen = 0;
	ch = getc(fp);
	while ((ch != INP_SEQ_SCH) && (ch != EOF)) {

		if ((ch != '\n') && (ch != COMMENT)) {
			line[0] = ch;
			fgets(&line[1], MAX_LINE_LEN-2, fp);
			len = remove_blanks(line, strlen(line), newLine);
			upper_case(newLine);

			if (TRUE == check_sbuf_line(newLine, len))
				for (i=0; i<len; i++)
					sBuf[sLen++] = AA_UC(newLine[i]);
	
			else {
				for (i=0; i<len; i++) {
					ch = AA_UC(newLine[i]);	/* convert to upper case */
					if (AA_CVALID(ch))
						pBuf[pLen++] = ch;
					else {
						sprintf(sVal, "Illegal input line: %s", line);
						show_err(TRUE, "read_seq_data", sVal);
					}
				}
			}
		
			if (pLen > MAX_RES_LEN) { /* too many residues */
				sprintf(sVal, "Sequence has > %d amino acids",
					MAX_RES_LEN);
				show_err(TRUE, "read_seq_data", sVal);
			}
		}
		ch = getc(fp);
	}

	pBuf[pLen] = '\0';
	sBuf[sLen] = '\0';
	ungetc(ch, fp);

}	/* read_seq_data */


/*************************************************************/
/* store the sequence data we read making sure it is ok.
*/
static void store_seq_data(rSeq, pBuf, sBuf)
SEQ_REC *rSeq;
char  *pBuf, *sBuf;
{
	register int i, pLen, sLen;
	char sVal[MAX_LINE_LEN];

	pLen = strlen(pBuf);
	sLen = strlen(sBuf);

	rSeq->numRes = pLen;

	if (pLen < MIN_SEQ_LEN) {
		sprintf(sVal, "Sequence has < %d amino acids.", MIN_SEQ_LEN);
		show_err(TRUE, "store_seq_data", sVal);
	}
	for (i=0; i<pLen; i++) {
		rSeq->pRes[i] = pBuf[i];
	}

	if (sLen > 0) {
		if (pLen != sLen) {
			sprintf(sVal, "Primary/secondary structures do not line up.");
			show_err(TRUE, "store_seq_data", sVal);
		}
		for (i=0; i<sLen; i++)
			rSeq->sRes[i] = sBuf[i];
	}

	else {
		for (i=0; i<pLen; i++)
			rSeq->sRes[i] = CH_X;
	}

}	/* store_seq_data */


/*************************************************************/
/* decide if an input line from the sequence file is a 
** secondary structure line.
*/
static int check_sbuf_line(line, len)
char *line;
int len;
{
	register int i;

	for (i=0; i<len; i++) {
		if (AA_SVALID(line[i]) == FALSE)
			return(FALSE);
	}

	return(TRUE);

}	/* check_sbuf_line */
