
/*
	file: rmolmat.c
	purpose: reading in molecular similarity matrix file 
*/

#include "defs.h" 

/* global variables */
char	molMatFname[MAX_FNAME_LEN];
MOL_MAT_REC	molMat[CODES][CODES];
int		molMatNum[CODES][CODES];

/* procedure calls */
static int read_molmat_file(), proc_atom_entry();
static void init_molmat_table();
extern void show_err();


/*************************************************************/
/* read molecular similarity matrix 
*/
void read_wt_mol(parmsfp)
FILE *parmsfp;
{
	FILE *fp;
	char sVal[MAX_LINE_LEN];
	int errs;

	if (read_prompt(parmsfp, PARMS_PROMPT) == FALSE) {
		sprintf(sVal, "Cannot find molecular similarity file\n");
		show_err(TRUE, "read_wt_mol", sVal);
	}

	fscanf(parmsfp, "%s\n", molMatFname);
	fp = fopen(molMatFname, "r");
	if (fp == NULL) {
		sprintf(sVal, "Cannot open molecular similarity file: %s\n",
			molMatFname);
		show_err(TRUE, "read_wt_mol", sVal);
	}

	fprintf(stdout, "Reading molecular similarity file: %s\n", molMatFname);
	errs = read_molmat_file(fp, molMat);
	if (errs > 0) {
		sprintf(sVal, "Number of errs found: %d\n", errs);
		show_err(FALSE, "read_wt_mol", sVal);
	}
	fclose(fp);

}	/* read_wt_mol */


/*************************************************************/
/* Reads molecular similarity table 
*/
static int read_molmat_file(fp, data)
FILE *fp;
MOL_MAT_REC	data[CODES][CODES];
{
	int rc, atomFlag, iAtom, jAtom, errs;
	char aStr[MAX_LINE_LEN], sVal[MAX_LINE_LEN];

	errs = 0;
	init_molmat_table(data);

	atomFlag = FALSE;
	while (TRUE) {
		if (EOF == fscanf(fp, "%s", aStr))
			break;
		if (TRUE == COMMENT_LINE(aStr[0]))
			fgets(aStr, sizeof(aStr), fp);
		else {
			if (aStr[1] == '-') {
				iAtom = AA_IX(AA_UC(aStr[0]));
				jAtom = AA_IX(AA_UC(aStr[2]));
				if (FALSE == AA_IVALID(iAtom)) {
					sprintf(sVal, "Unrecognized amino acid: %s\n", aStr[0]);
					show_err(FALSE, "read_molmat_file", sVal);
					errs++;
				}
				if (FALSE == AA_IVALID(jAtom)) {
					sprintf(sVal, "Unrecognized amino acid: %s\n", aStr[2]);
					show_err(FALSE, "read_molmat_file", sVal);
					errs++;
				}
				atomFlag = TRUE;
				if (molMatNum[iAtom][jAtom] > 0)
					atomFlag = TRUE;
			}

			else if (atomFlag == TRUE) {
				rc = proc_atom_entry(data, iAtom, jAtom, aStr);
				if (rc == ERROR_FLAG)
					errs++;
			}

			else {
				sprintf(sVal, "Cannot recognize format: %s\n", aStr);
				show_err(FALSE, "read_molmat_file", sVal);
				errs++;
			}
		}
	}

	if (FALSE == atomFlag) {
		sprintf(sVal, "No entries, check mol data file\n");
		show_err(TRUE, "read_molmat_file", sVal);
	}

	return(errs);

}	/* read_molmat_file */


/*************************************************************/
/* process one atom:atom entry in table 
*/
static int proc_atom_entry(data, iAtom, jAtom, aStr)
MOL_MAT_REC	data[CODES][CODES];
int iAtom, jAtom;
char *aStr;
{
	int rc, atom1, atom2, scor, count;
	char *s1, *sAtom1, *sAtom2, bStr[MAX_LINE_LEN], sVal[MAX_LINE_LEN];

	strcpy(bStr, aStr);

/* get atom names */

	sAtom1 = strtok(aStr, ":");
	if (sAtom1 == NULL) {
		sprintf(sVal, "Unrecognized atom 1 format at: %c-%c  %s\n", 
			AA_CH(iAtom), AA_CH(jAtom), bStr);
		show_err(FALSE, "proc_atom_entry", sVal);
		return(ERROR_FLAG);
	}
	sAtom2 = strtok((char *)NULL, "=");
	if (sAtom2 == NULL) {
		sprintf(sVal, "Unrecognized atom 2 format at: %c-%c  %s\n", 
			AA_CH(iAtom), AA_CH(jAtom), bStr);
		show_err(FALSE, "proc_atom_entry", sVal);
		return(ERROR_FLAG);
	}

/* look for atom/atom scor */

	scor = MAX_ATOM_SCOR;
	s1 = strtok((char *)NULL, "=");
	if (s1 != NULL) {
		scor = atoi(s1) * 10;
	}

	rc = ERROR_FLAG;

/* record atom assignment score */

	atom1 = atom_to_index(iAtom, sAtom1);
	atom2 = atom_to_index(jAtom, sAtom2);
	if ((atom1 >= 0) && (atom2 >= 0)) {
		count = molMatNum[iAtom][jAtom];
		data[iAtom][jAtom].atom1[count] = atom1;
		data[iAtom][jAtom].atom2[count] = atom2;
		data[iAtom][jAtom].scor[count] = scor;
		molMatNum[iAtom][jAtom] = count+1;
		rc = 0;
	}

/* assign the reverse atom pairing if valid */

	if (0 != strcmp(sAtom1, sAtom2)) {
		atom1 = atom_to_index(iAtom, sAtom2);
		atom2 = atom_to_index(jAtom, sAtom1);
		if ((atom1 >= 0) && (atom2 >= 0)) {
			count = molMatNum[iAtom][jAtom];
			data[iAtom][jAtom].atom1[count] = atom1;
			data[iAtom][jAtom].atom2[count] = atom2;
			data[iAtom][jAtom].scor[count] = scor;
			molMatNum[iAtom][jAtom] = count+1;
			rc = 0;
		}
	}

/* assign the reverse amino acid pairing if different */

	if (iAtom != jAtom) {
		atom1 = atom_to_index(jAtom, sAtom1);
		atom2 = atom_to_index(iAtom, sAtom2);
		if ((atom1 >= 0) && (atom2 >= 0)) {
			count = molMatNum[jAtom][iAtom];
			data[jAtom][iAtom].atom1[count] = atom1;
			data[jAtom][iAtom].atom2[count] = atom2;
			data[jAtom][iAtom].scor[count] = scor;
			molMatNum[jAtom][iAtom] = count+1;
			rc = 0;
		}

/* assign the reverse atom pairing if valid */

		if (0 != strcmp(sAtom1, sAtom2)) {
			atom1 = atom_to_index(jAtom, sAtom2);
			atom2 = atom_to_index(iAtom, sAtom1);
			if ((atom1 >= 0) && (atom2 >= 0)) {
				count = molMatNum[jAtom][iAtom];
				data[jAtom][iAtom].atom1[count] = atom1;
				data[jAtom][iAtom].atom2[count] = atom2;
				data[jAtom][iAtom].scor[count] = scor;
				molMatNum[jAtom][iAtom] = count+1;
				rc = 0;
			}
		}
	}

	if (rc == ERROR_FLAG) {
		sprintf(sVal, "Unrecognized atom pairing at: %c-%c  %s\n", 
			AA_CH(iAtom), AA_CH(jAtom), bStr);
		show_err(FALSE, "proc_atom_entry", sVal);
		return(ERROR_FLAG);
	}
	return(0);

}	/* proc_atom_entry */


/*************************************************************/
/* init molecular similarity matrix.
*/
static void init_molmat_table(data)
MOL_MAT_REC	data[CODES][CODES];
{
	register int i, j, k, count;

	for (i=0; i<CODES; i++) {
		for (j=0; j<CODES; j++) {
			molMatNum[i][j] = 0;
		}
	}

}	/* init_molmat_table */
