
/*
	file...: seqcalc.c 
	purpose: calculations based on sequence information

	author.: Robert Boyko
	date...: May 25, 1995	
	place..: University of Alberta / Department of Biochemistry
*/

#include <math.h>
#include "defs.h"

/* procedure calls */

static void calc_hom_glob_scor(), calc_hom_loc_scor(), calc_hom_struct_scor();
double calc_hom_ave_scor();
extern void show_err();
extern double pow();


/*************************************************************/
/* calculate weights from parameters for each homologous sequence
*/
void calc_parm_weights()
{
	register int i;

/* get weights for the individual factors */

    for (i=0; i<numSeq; i++) {
		if (i != seqSelect) {
			calc_hom_glob_scor(i);
			calc_hom_loc_scor(i);
			calc_hom_struct_scor(i);
		}
	}


}	/* calc_p0_parm */


/*************************************************************/
/* calc global homology for known and unknown sequence 
*/
static void calc_hom_glob_scor(seqId)
int seqId;
{

	register int k, scor, firstRes, lastRes;
	char ch1, ch2, sVal[MAX_LINE_LEN];
	int s1, e1, s2, e2, soff, eoff;
	double best;

	scor = 0;
	best = 1;

	s1 = seqArr[seqSelect].startIx;
	e1 = seqArr[seqSelect].endIx;
	s2 = seqArr[seqId].startIx;
	e2 = seqArr[seqId].endIx;

	firstRes = MAX(s1, s2);
	soff = abs(s2 - s1);
	lastRes = MIN(e1, e2);
	eoff = abs(e2 - e1);

	if ((firstRes == s1) && (lastRes == e2)) {
		if (soff < eoff)
			firstRes = s2; 
		else
			lastRes = e1;
	}

	if ((lastRes == e1) && (firstRes == s2)) {
		if (soff < eoff)
			lastRes = e2; 
		else
			firstRes = s1;
	}

	for (k=firstRes; k<=lastRes; k++) {

		ch1 = seqArr[seqSelect].residue[k].pRes;
		ch2 = seqArr[seqId].residue[k].pRes;

		if ((TRUE == AA_CVALID(ch1)) && (TRUE == AA_CVALID(ch2))) {
			scor += wtHomolMat[(AA_IX(ch1) * WT_MAT_DIM) + AA_IX(ch2)];
			best += wtHomolMat[(AA_IX(ch1) * WT_MAT_DIM) + AA_IX(ch1)];
		}

		else if (TRUE == AA_CVALID(ch1))
			best += wtHomolMat[(AA_IX(ch1) * WT_MAT_DIM) + AA_IX(ch1)];

		else if (TRUE == AA_CVALID(ch2))
			best += wtHomolMat[(AA_IX(ch2) * WT_MAT_DIM) + AA_IX(ch2)];
	}

	if (best <= 0) {
		sprintf(sVal, "Check weight matrix, best score <= 0\n", best);
		show_err(TRUE, "calc_global_hom", sVal);
	}

	if (scor < 0) {
		sprintf(sVal, "Check weight matrix, homology score < 0\n", scor);
		show_err(TRUE, "calc_global_hom", sVal);
	}

	seqArr[seqId].homGlobScor = (int)(0.5 + (scor * 100.0) / best);

}	/* calc_hom_glob_scor */


/*************************************************************/
/* calc local homology  
*/
static void calc_hom_loc_scor(seqId)
int seqId;
{
	register int k, counter, scor, hix, windSize;
	char ch1, ch2;
	double best;

	windSize = csLocWind;

	for (k=0; k<alignLen; k++) {

		hix = k - (windSize / 2);
		scor = 0;
		best = 0;
		counter = 0;

		while (counter < windSize) {

			if ((hix > seqArr[seqId].endIx) || (hix < seqArr[seqId].startIx)) {
				best += wtHomolMat[(AA_IX('A') * WT_MAT_DIM) + AA_IX('A')];
			}

			else {

				ch1 = seqArr[seqSelect].residue[hix].pRes;
				ch2 = seqArr[seqId].residue[hix].pRes;

				if ((TRUE == AA_CVALID(ch1)) && (TRUE == AA_CVALID(ch2))) {
					scor += wtHomolMat[(AA_IX(ch1) * WT_MAT_DIM) + AA_IX(ch2)];
					best += wtHomolMat[(AA_IX(ch1) * WT_MAT_DIM) + AA_IX(ch1)];
				}

				else if (TRUE == AA_CVALID(ch1))
					best += wtHomolMat[(AA_IX(ch1) * WT_MAT_DIM) + AA_IX(ch1)];

				else if (TRUE == AA_CVALID(ch2))
					best += wtHomolMat[(AA_IX(ch2) * WT_MAT_DIM) + AA_IX(ch2)];

				else
					counter--;
			}

			counter++;
			hix++;
		}

		if ((best <= 0) || (scor < 0))
			seqArr[seqId].residue[k].homLocScor = ERROR_FLAG;
		else
			seqArr[seqId].residue[k].homLocScor = (int)
				(0.5 + (scor * 100.0) / best);

	}

}	/* calc_hom_loc_scor */


/*************************************************************/
/* calc local structural homology  
*/
static void calc_hom_struct_scor(seqId)
int seqId;
{
	register int k, counter, scor, hix, windSize;
	char ch1, ch2;
	double best;

	windSize = csStructWind;

	for (k=0; k<alignLen; k++) {

		hix = k - (windSize / 2);
		scor = 0;
		best = 0;
		counter = 0;

		while (counter < windSize) {

			if (hix > seqArr[seqId].endIx)
				break;

			if (hix >= seqArr[seqId].startIx) {

				ch1 = seqArr[seqSelect].residue[hix].sRes;
				ch2 = seqArr[seqId].residue[hix].sRes;

				if ((TRUE == AA_SVALID(ch1)) || (TRUE == AA_SVALID(ch2)))
					counter++;

				if ((TRUE == AA_SVALID(ch1)) && (TRUE == AA_SVALID(ch2))) {
					scor += wtStructMat[(AA_IX(ch1) * WT_MAT_DIM) + AA_IX(ch2)];
					best += wtStructMat[(AA_IX(ch1) * WT_MAT_DIM) + AA_IX(ch1)];
				}

				else if (TRUE == AA_SVALID(ch1))
					best += wtStructMat[(AA_IX(ch1) * WT_MAT_DIM) + AA_IX(ch1)];

				else if (TRUE == AA_SVALID(ch2))
					best += wtStructMat[(AA_IX(ch2) * WT_MAT_DIM) + AA_IX(ch2)];

			}

			hix++;
		}

		if ((best <= 0) || (scor < 0))
			seqArr[seqId].residue[k].homStructScor = ERROR_FLAG;
		else
			seqArr[seqId].residue[k].homStructScor = (int)
				(0.5 + (scor * 100.0) / best);

	}

}	/* calc_hom_struct_scor */


/*************************************************************/
/* calc the weighted average homology score 
*/
double calc_hom_ave_scor(molScor, globScor, locScor, structScor)
int molScor, globScor, locScor, structScor;
{
	double wi;

/* all scors meet minimum requirements */

	if ((globScor < csGlobMin) || (structScor < csStructMin) ||
		(locScor < csLocMin) || (molScor < csMolMin))
		return(0.0);

	wi = 0.0000001 * molScor * globScor * locScor * structScor;
	wi = pow(2.0, wi);
	return(wi);

}	/* calc_hom_ave_scor */
