
######################################################################
#
# read shifts file, the shifts file determines if an assignment is
# reasonable
#
proc data_read_shifts { fn } {
	global snbVar snbConst mySeq

	if {[ catch { set shiftsFile [open $fn r ] } err]} {
		usnb_err "Cannot read shift constraints file '$fn'"
	}

	set aaIx 0
	set curId -99
	set dggCurrent 0
	set mySeq(dgg,num) 0

	while {[gets $shiftsFile line] >= 0} {
		set first [string index $line 0]
		if { $first != $snbConst(COMMENT) && $first != "!" } {
			set nd [scan $line "%s %s %s" s1 mean sdev]

			# parse string for amino acid id number and atom name
			set iPos [string last ":" $s1]
			set aaNum [string range $s1 6 [expr $iPos - 1]]
			set aaName [ string range $s1 2 4 ]
			set atomName [string range $s1 [incr iPos] end]
			set atomName [ string tolower $atomName ]

			# is this a new amino acid?
			if { $curId != $aaNum } {
				incr aaIx
				set curId $aaNum
				set mySeq($aaNum,aaIx) $aaIx
				set mySeq($aaIx,aaNum) $aaNum
				set mySeq($aaIx,name3) [ string toupper $aaName ]
				set mySeq($aaIx,name1) [ u_resname_3to1 $mySeq($aaIx,name3) ]
				set mySeq($aaIx,assignChainIx) -1

				# take note of any GLY/GLY residue pair in the sequence
				if { $mySeq($aaIx,name1) == "G" } {
					if { $dggCurrent == 1 } {
						incr mySeq(dgg,num)
					} else {
						incr dggCurrent
					}
				} else {
					set dggCurrent 0
				}
			}

			if ![info exists mySeq($aaIx,resAtoms)] {
				set mySeq($aaIx,resAtoms) $atomName
			} else {
				if {[lsearch $mySeq($aaIx,resAtoms) $atomName] < 0} {
					lappend mySeq($aaIx,resAtoms) $atomName
				}
			}

			# make a record of the shift we just read
			set mySeq($aaIx,$atomName,mean) $mean
			set mySeq($aaIx,$atomName,sdev) $sdev
		}
	}
	set mySeq(seqLen) $aaIx
	close $shiftsFile
}

######################################################################
#
# read current ppm assignments file
# the user may be using smartnotebook with assignments already made
#
proc data_read_ppm { fn assignType } {
	global snbVar snbConst mySeq

	if {[ catch { set ppmFile [ open $fn r ] } err]} {
		return
	}

	while {[gets $ppmFile line] >= 0} {
		if { [string index $line 0] != $snbConst(COMMENT)} {
			set nd [scan $line "%s %s %d" s1 s2 ambig]
			set iPos [string last "." $s1]
			set aaNum [string range $s1 0 [expr $iPos - 1]]
			set atomName [string range $s1 [incr iPos] end]
			set atomName [ string tolower $atomName ]
			set aaIx $mySeq($aaNum,aaIx)

			if ![info exists mySeq($aaIx,resAtoms)] {
				set mySeq($aaIx,resAtoms) $atomName
			} else {
				if {[lsearch $mySeq($aaIx,resAtoms) $atomName] < 0} {
					lappend mySeq($aaIx,resAtoms) $atomName
				}
			}
			set mySeq($aaIx,$atomName,$assignType) $s2
			set mySeq($aaIx,$atomName,ambig) $ambig
		}
	}
	close $ppmFile
}

######################################################################
#
# write ppm file, this includes the ppm file originally read in plus
# any assignments made while using smartnotebook.
#
proc data_write_ppm { fn } {
	global snbVar snbConst mySeq

	if {[ catch { set fppm [open $fn w ] } err]} { 
		usnb_err "Cannot open '$fn' for writing"
	}

	set count 0
	for {set i 1} {$i <= $mySeq(seqLen)} {incr i} {
		foreach j $mySeq($i,resAtoms) {

			# assigned shifts within snb take precedence
			if [info exists mySeq($i,$j,snbAssign)] {
				if { $mySeq($i,assignChainIx) >= 0 } {
					set s1 [format "%3d.%-5s" $i $j]
					set s2 [format "%8.2f %d" $mySeq($i,$j,snbAssign) 1]
					puts $fppm "$s1 $s2"
					incr count
				}
			} else {
				if [info exists mySeq($i,$j,ppmAssign)] {
					set s1 [format "%3d.%-5s" $i $j]
					if [info exists mySeq($i,$j,ambig)] {
						set ambig $mySeq($i,$j,ambig) 
					} else {
						set ambig 1
					}
					set s2 [format "%8.2f %d" $mySeq($i,$j,ppmAssign) $ambig]
					puts $fppm "$s1 $s2"
					incr count
				}
			}
		}
	}
	close $fppm
	usnb_log file "Wrote $count ppm.out records to '$fn'"
}

######################################################################
#
# read all the xpk files
#
proc data_read_xpk { } {
	global snbVar snbConst myPeak mySpectra

	set fileList ""
	for {set i 0} {$i < $mySpectra(xpk,fileNum)} {incr i} {
		if { [lsearch -exact $fileList $mySpectra(xpk,file,$i)] < 0 } {
			read_xpk_file $i $mySpectra(xpk,file,$i)
			lappend fileList $mySpectra(xpk,file,$i)
		}
	}
}

######################################################################
#
# read one xpk file (a peaklist file). 
#
proc read_xpk_file { xpkix filename } {
	global snbVar snbConst myPeak mySpectra

	set ATOM  2
	set UB    4
	set LB    5

	if { [ catch { set xpkFile [open $filename r ] } err] } {
		usnb_err "Cannot read xpk file '$filename'"
	}

	set iHead 0
	set iPeak 0
	foreach j $mySpectra(xpk,fields,$xpkix) {
		set myPeak($xpkix,ave,$j) 0.0
	}

	while {[gets $xpkFile line] >= 0} {
		if { [string index $line 0] == $snbConst(COMMENT) } {
			continue
		}
		set pkId [lindex $line 0]

		# check for the first argument being an integer
		if {[ catch { set a [expr $pkId & $pkId] } err]} {
			incr iHead 
			set myPeak($xpkix,header,$iHead) $line
		} else {
			incr iPeak 
			set myPeak($xpkix,index,$pkId) $iPeak
			set myPeak($xpkix,pkId,$iPeak) $pkId
			set myPeak($xpkix,origLine,$iPeak) $line
			set myPeak($xpkix,origPeak,$iPeak) [usnb_peak_get $line $mySpectra(xpk,fields,$xpkix)]
			foreach j $mySpectra(xpk,fields,$xpkix) {
				set aVal [lindex $line [expr $j - 1]]
				set myPeak($xpkix,ave,$j) [expr $myPeak($xpkix,ave,$j)+$aVal]
			}
		}
	}
	close $xpkFile
	set myPeak($xpkix,nPeak) $iPeak
	set myPeak($xpkix,nHead) $iHead

	# get sweep width in each dimension
	set myPeak($xpkix,atomNames) $myPeak($xpkix,header,$ATOM)
	set count 0
	foreach ub $myPeak($xpkix,header,$UB) {
		set lb [lindex $myPeak($xpkix,header,$LB) $count]
		lappend sweep [expr $ub / $lb]
		incr count
	}
	set myPeak($xpkix,sw) $sweep 

	# get average value for all shifts
	foreach j $mySpectra(xpk,fields,$xpkix) {
		set myPeak($xpkix,ave,$j) [expr $myPeak($xpkix,ave,$j) / $iPeak] 
	}
}

######################################################################
#
# write out all xpk files. 
#
proc data_write_xpk { fileNames } {
	global snbVar snbConst myPeak

	set count 0
	set fileList ""
	foreach xpk $fileNames {
		if { [lsearch -exact $fileList $xpk] < 0 } {
			write_xpk_file $count $xpk
			lappend fileList $xpk
			incr count
		}
	}
}

######################################################################
#
# write out one xpk file (a peaklist file). 
#
proc write_xpk_file { xpkix filename } {
	global snbVar snbConst myPeak

	if { [ catch { set fp [open $filename w ] } err] } {
		usnb_err "Cannot write xpk file '$filename'"
	}

	# write header lines first
	for {set iPeak 1} {$iPeak <= $myPeak($xpkix,nHead)} {incr iPeak} {
		puts $fp $myPeak($xpkix,header,$iPeak) 
	}

	# write each xpk line, make sure atom names are in capital letters
	for {set iPeak 1} {$iPeak <= $myPeak($xpkix,nPeak)} {incr iPeak} {
		if [info exists myPeak($xpkix,assignLine,$iPeak)] {
			set line [string toupper $myPeak($xpkix,assignLine,$iPeak)]
		} else { 
			if [info exists myPeak($xpkix,foldedLine,$iPeak)] {
				set line [string toupper $myPeak($xpkix,foldedLine,$iPeak)]
			} else {
				set line [string toupper $myPeak($xpkix,origLine,$iPeak)]
			}
		}
		puts $fp $line
	}
	close $fp
	usnb_log file "Wrote $myPeak($xpkix,nPeak) peaklist records to '$filename'"
}

######################################################################
#
# read connectivity files
#
proc data_read_connect { } {
	global snbVar snbConst myConnect

	# start fresh
	if [ info exists myConnect ] {
		unset myConnect
	}

	set myConnect(c_rules) ""
	set myConnect(connectNum) 0 
	set myConnect(refIdMax) 0

	# get names of the connect files
    set outStr $snbVar(fn,session)/$snbVar(parm,c_out)*
	if { ![ catch { set outFiles [glob $outStr] } err]} {
		set myConnect(filenames) [lsort $outFiles]
	} else {
		usnb_log all " "
		usnb_log all "No connectivity files found in $outStr."
		usnb_log all "I assume you will be importing your own..."
		usnb_log all " "
		return
	}

	# read each file
	foreach fn $myConnect(filenames) { 
		set count [read_connect_file $fn]
		set msg [format "  Found %5d connections in '%s'" $count $fn]
		usnb_log all $msg
	}
}

######################################################################
#
# read one connectivity file
#
proc read_connect_file { fn } {
	global snbVar snbConst myConnect mySpectra

	set cix $myConnect(connectNum) 
	set x0 $mySpectra(connect,REF,0)
	set x1 $mySpectra(connect,REF,1)
	set iProb $mySpectra(connect,PROB)
	set iRule $mySpectra(connect,RULE)

	set fp [open $fn r]
	set count 0
	while {[gets $fp line] >= 0} {
		if { [string index $line 0] == $snbConst(COMMENT)} {
			continue
		}
		set prob [lindex $line $iProb]
		if { $prob < 0.0 || $prob > 1.0} {
			set msg $fn:$line
			usnb_log all "Warning, check connection file format: $msg"
			continue
		}

		set ref0 [lindex $line $x0]
		set ref1 [lindex $line $x1]
		set rule [lindex $line $iRule]

		# keep list of rule types 
		if { [string first $rule $myConnect(c_rules)] < 0 } {
			set myConnect(c_rules) "$myConnect(c_rules) $rule"
		}

		# do we need to create a new connection record?
		if ![info exists myConnect(index,$rule,$ref0,$ref1)] {
			incr cix
			connect_create_ref $cix $rule $ref0 $ref1
			foreach peakId "$ref0 $ref1" {
				if { $peakId > $myConnect(refIdMax) } {
					set myConnect(refIdMax) $peakId
				}
			}
		}

		connect_create_item $cix $line $snbConst(C_OK)
		incr count
	}
	close $fp
	set myConnect(connectNum) $cix
	return $count
}

######################################################################
#
# read in user created chains
#
proc data_read_chains { fn } {
	global snbVar snbConst myChain myConnect

	# empty the chain status for each reference ID
	for {set ref0 0} {$ref0 <= $myConnect(refIdMax)} {incr ref0} {
		set myChain($ref0,chainId) -1
	}
	set myChain(chainNum) 0

	read_chains $fn "TRUE"

	# if there are no chains, then start off by initializing a chain 
	if { $myChain(chainNum) == 0 } {
		chains_init
	}
}

######################################################################
#
# read a chains file 
#
proc read_chains { fn updateAssign } {
	global snbVar snbConst myChain myConnect

	set myChain(current) 0

	# if file doesn't exist, then nothing to do
	if ![file exists $fn] { return }

	# file can not be open for reading
	if {[ catch { set fp [open $fn r ] } err]} { 
		usnb_err "Cannot open chains file '$fn', check permissions"
	}

	# read first line to find out the user's current display chain
	while {[gets $fp line] >= 0} {
		if { [string index $line 0] != $snbConst(COMMENT)} {
			set myChain(current) [lindex $line 0]
			break
		}
	}

	# read chains we have previously made
	set cur 0
	while {[gets $fp line] >= 0} {
		if { [string index $line 0] == $snbConst(COMMENT)} {
			continue
		}
		incr cur
		set myChain($cur,aaIx) [lindex $line 0 ]
		set myChain($cur,nConnect) [lindex $line 1 ] 
		set myChain($cur,cLabel) [lindex $line 2] 

		# now read each connection record within a single chain
		set aaIx $myChain($cur,aaIx)
		set count 1
		while { $count <= $myChain($cur,nConnect) } {
			if {[gets $fp line] < 0} {
				usnb_err "Unexpected EOF at chains file '$fn', chain $count."
			}
			if { [string index $line 0] == $snbConst(COMMENT)} {
				continue
			}
			set ref0 [lindex $line 0 ]
			set ref1 [lindex $line 1 ]
			set rule [lindex $line 2 ]

			# find and save the index of connections data structure 
			if [info exists myConnect(index,$rule,$ref0,$ref1)] {
				set cix $myConnect(index,$rule,$ref0,$ref1)
			} else {
				usnb_log all "Warning, no connection record for\n $rule $ref0 $ref1 \n"
				incr myConnect(connectNum)
				set cix $myConnect(connectNum)
				connect_create_ref $cix $rule $ref0 $ref1
			}

			# determine which connection record variation has matching info 
			set pix [usnb_get_pix $line $cix ]
			if { $pix == $snbConst(NO_VAL) } {
				usnb_log all "Warning, no connection record variation for\n $line\n"
				# fake a connection record just because it may be
				# interesting to keep track of this info.
				connect_create_item $cix $line $snbConst(NO_VAL)
				set pix $myConnect($cix,pNum)
			}
			set myChain($cur,$count,cix) $cix
			set myChain($cur,$count,pix) $pix 

			# record this refId as being used in this chain 
			lappend myChain($ref0,chainId) $cur
			incr count 
		}

		# don't forget to record second ref ID of last connection
		lappend myChain($ref1,chainId) $cur

		# do assignment stuff if chain has been assigned 
		if { $aaIx >= 0 && $updateAssign == "TRUE" } {
			assign_add_update $cur $aaIx
		}
	}
	close $fp

	# this is how many chains we have currently 
	set myChain(chainNum) $cur
}

######################################################################
#
# backup chains before writing out 
#
proc data_write_chains { fn } {
	global snbVar snbConst

	if [file exists $snbVar(fn,chains)] {
		usnb_log file "\n*** Backup File: '$snbVar(fn,chains)'"
		usnb_backup_file $snbVar(fn,chains) $snbVar(fn,log)
		usnb_log file "*** End Backup File: '$snbVar(fn,chains)'\n"
	}
	write_chains $fn
}

######################################################################
#
# write current user chains to chains file 
#
proc write_chains { fn } {
	global snbVar snbConst myChain myConnect mySpectra

	# make sure we have correct current chain after removing any null chains
	set currentChain $myChain(current)
	for {set i 1} {$i <= $myChain(current)} {incr i} {
		set numConnect $myChain($i,nConnect)
		if { $numConnect <= 0 } {
			set currentChain [expr $currentChain - 1]
		}
	}

	# open our chains file for writing
	if {[ catch { set fp [open $fn w ] } err]} { 
		usnb_err "Cannot not open '$fn' for writing"
	}

	puts $fp "#"
	puts $fp "# This file keeps track of chains made by the user"
	puts $fp "# Smartnotebook version: $snbVar(VERSION)"
	puts $fp "#"
	puts $fp "# RefId(i-1) RefId(i)  Pattern Fit  Relevant-connectivity-peaks"
	puts $fp "#"
	puts $fp "$currentChain (current chain user last selected)"

	# write out the valid chains
	set num $myChain(chainNum)
	for {set i 1} {$i <= $num} {incr i} {
		set numConnect $myChain($i,nConnect)
		if { $numConnect > 0 } {
			set line [format "%-3d %3d   %s" $myChain($i,aaIx) $numConnect $myChain($i,cLabel)]
			puts $fp $line 
			for {set j 1} {$j <= $numConnect} {incr j} {
				set cix $myChain($i,$j,cix)
				set line [format " %3d %3d %s" $myConnect($cix,ref,0) \
					$myConnect($cix,ref,1) $myConnect($cix,rule) ]

				# use the information found in the connection record.
				set pix $myChain($i,$j,pix)
				set iProb $mySpectra(connect,PROB)
				if [info exists myConnect($cix,$pix,$iProb)] {
					set item [format "%5.2f" $myConnect($cix,$pix,$iProb)]
				} else {
					set item [format "%5.2f" $snbConst(NO_VAL)]
				}
				set line "$line $item   "
				foreach resInc "0 1" {
					foreach field $mySpectra(connect,PKID,$resInc) {
						if [info exists myConnect($cix,$pix,$field)] {
							set item [format "%3d" $myConnect($cix,$pix,$field)]
						} else {
							set item [format "%3d" $snbConst(NO_VAL)]
						}
						set line "$line $item"
					}
				}
				puts $fp $line
			}
		}
	}
	close $fp
}
